def format_columns(*items, **opts):
    if not items:
        return

    width = [
        max(map(len, [(items[row][column] or '').strip()
                      for row in range(len(items))]))
        for column in range(len(items[0]))
    ]

    line_delim = opts.get('line_delim', '\n')
    column_delim = opts.get('column_delim', '  ')
    line_prefix = opts.get('line_prefix', ' ')

    return line_delim.join([
        ''.join([
            line_prefix,
            column_delim.join([
                (value or '').ljust(width[column])
                for column, value in enumerate(row)
            ])
        ])
        for row in items
    ])



from __future__ import unicode_literals

from ds import context


class ContainerNaming(context.Context):
    ensure_container_name = True

    def check(self):
        super(ContainerNaming, self).check()
        if self.ensure_container_name:
            assert (self.has_container_name and self.container_name), \
                'Container name is not defined'

    @property
    def has_container_name(self):
        return hasattr(self, 'container_name')

    @property
    def has_image_name(self):
        return hasattr(self, 'image_name')


class ImageNaming(ContainerNaming):
    def check(self):
        super(ImageNaming, self).check()
        assert (self.has_image_name and self.image_name), \
            'Image name is not defined'


class DefaultNaming(ImageNaming, ContainerNaming):
    @property
    def container_name(self):
        if not self.image_name:
            return
        image = self.image_name.split(':', 1)[0]
        return '--'.join([self.project_name, image])


class BuildNaming(ContainerNaming):
    def check(self):
        assert self.image_name, 'Image suffix is not defined'
        super(BuildNaming, self).check()

    @property
    def image_name(self):
        if not self.image_suffix:
            return
        return '/'.join([self.project_name, self.image_suffix])

    @property
    def container_name(self):
        if not self.image_name:
            return
        return self.image_name.replace('/', '--')

    @property
    def image_suffix(self):
        return self.__module__



import os
import sys
from logging import getLogger
from os import getcwd
from os.path import join

from ds.command import Command
from ds.context import Context as _Context
from ds.environment import get_environment
from ds.summary import TableSummary
from ds.utils import flatten, drop_empty


logger = getLogger('compose')


class Context(_Context):
    compose_bin = 'docker-compose'
    compose_verbose = False

    logs_default = (
        '--follow',
        '--tail=100',
    )

    default_shell = '/bin/bash'

    def get_commands(self):
        return super(Context, self).get_commands() + [
            Compose,
            Switch,
            Config,
            Build,
            Up,
            Down,
            Start,
            Stop,
            Rm,
            Logs,
            Ps,
            Restart,
            Kill,
            Exec,
            Shell,
            RootShell,
            Recreate,
        ]

    def get_environment_variables(self, **kwargs):
        result = {
            'project_root': self.project_root,
            'project_name': self.project_name,
            'user': os.getuid(),
        }
        result.update(kwargs)
        return result

    @property
    def environment_variables(self):
        return self.get_environment_variables()

    def get_compose_files(self):
        return 'compose.yml',

    @property
    def compose_files(self):
        return self.get_compose_files()

    def get_current_services(self):
        return get_environment().get('current_services', None)

    def set_current_services(self, value):
        logger.debug('Set services: %s', value)
        get_environment().set('current_services', value)

    current_services = property(get_current_services, fset=set_current_services)

    @property
    def all_services(self):
        self.invoke_compose((
            'config',
            '--services',
        ))
        result = self.executor.commit()
        return result.stdout.strip().split()

    def get_compose_options(self):
        return (
            self.compose_bin,
            '--verbose' if self.compose_verbose else (),
            ('--project-directory', self.project_root),
            ('--project-name', self.project_name),
            [
                ('-f', filename)
                for filename in self.compose_files
            ],
        )

    @property
    def env_filename(self):
        return join(self.project_root, '.env')

    def update_env_file(self):
        with open(self.env_filename, 'w') as output:
            output.writelines([
                '{}={}\n'.format(k, v)
                for k, v in self.environment_variables.items()
            ])

    def invoke_compose(self, args):
        self.update_env_file()
        options = drop_empty(flatten((
            self.get_compose_options(),
            args,
        )))
        self.executor.append(options)

    def switch_current_service(self, allow_all=False):
        all_option = '(all)'
        services = self.all_services

        if allow_all:
            services = [all_option] + services

        result = self.executor.fzf(services, multi=allow_all)
        if not result:
            return

        if not allow_all:
            result = (result, )

        if all_option in result:
            result = None

        self.current_services = result

        return result

    def get_additional_summary(self):
        cells = [
            ['Files', repr(self.compose_files)],
            ['Services', repr(self.current_services or '(all)')],
        ]
        return super(Context, self).get_additional_summary() + [
            TableSummary('Compose', cells),
        ]

    def check(self):
        assert getcwd() == self.project_root, \
            'Change current directory to project root'
        super(Context, self).check()


class Switch(Command):
    consume_all_args = True

    def invoke_with_args(self, args):
        if not args:
            self.context.switch_current_service(allow_all=True)
            return

        services = set(self.context.all_services)
        if set(args) != {'.'}:
            services &= set(args)
        self.context.current_services = list(services)


class Recreate(Command):
    consume_all_args = False

    def invoke_with_args(self, args):
        logger.info('Rebuild')
        self.context['build']()

        logger.info('Stop')
        self.context['stop']()

        logger.info('Remove')
        self.context['rm']()

        logger.info('Up and detach')
        self.context['up']('-d')

        logger.info('Logs')
        self.context['logs']()


class ComposeCommand(Command):
    consume_all_args = True
    compose_command = None
    allow_multiple = True
    with_services = True
    prepend_args = False
    with_command_name = True

    def get_services(self):
        services = self.context.current_services
        if not services and not self.allow_multiple:
            services = self.context.switch_current_service()
            if services is None:
                sys.exit(1)
        return filter(lambda value: value, services or ())

    def get_command_options(self, args):
        return args if self.prepend_args else ()

    def get_command_args(self, args):
        return args if not self.prepend_args else ()

    def invoke_with_args(self, args):
        options = drop_empty(flatten((
            (self.compose_command or self.get_name()) if self.with_command_name else (),
            self.get_command_options(args),
            self.get_services() if self.with_services else (),
            self.get_command_args(args),
        )))
        self.context.invoke_compose(options)


class Compose(ComposeCommand):
    with_services = False
    with_command_name = False


class Config(ComposeCommand):
    with_services = False
    prepend_args = False


class Build(ComposeCommand):
    pass


class Up(ComposeCommand):
    prepend_args = True


class Down(ComposeCommand):
    pass


class Rm(ComposeCommand):
    pass


class Logs(ComposeCommand):
    prepend_args = True

    def get_command_options(self, args):
        return args or self.context.logs_default


class Ps(ComposeCommand):
    with_services = False


class Restart(ComposeCommand):
    pass


class Kill(ComposeCommand):
    pass


class Start(ComposeCommand):
    allow_multiple = False


class Stop(ComposeCommand):
    allow_multiple = False


class Exec(ComposeCommand):
    compose_command = 'exec'
    allow_multiple = False


class Shell(Exec):
    user = None

    def get_command_args(self, args):
        return args or self.context.default_shell,

    def get_command_options(self, args):
        return (
            ('--user={}'.format(self.user)) if self.user is not None else (),
        )


class RootShell(Shell):
    user = 0



import pytest

from ds.environment import get_environment
from ds.executor import Executor


class Shell(object):
    def __init__(self):
        self._executor = Executor()

    def call(self, *args, **options):
        get_environment().invalidate()
        self._executor.append(args, **options)
        return self._executor.commit()

    def call_ds(self, *args, **options):
        return self.call(*(('python', '-m', 'ds') + args), **options)

    def call_docker_ds(self, *args, **options):
        options.setdefault('skip_stdin', True)
        return self.call_ds(*args, **options)


@pytest.fixture(scope='module')
def shell():
    yield Shell()



from __future__ import unicode_literals
import sys
from functools import partial
from logging import getLogger

import six
try:
    import libtmux
except ImportError:
    print('Install libtmux with `pip install libtmux`')
    sys.exit(1)

from ds import context
from ds.command import Command
from ds.command import preset_base_command

logger = getLogger(__name__)


class SchemaRealizer(object):
    def realize(self, server, context):
        session = server.new_session(session_name=context.session_name)
        for key, value in context.envs.items():
            session.set_environment(key, value)

        windows = context.schema
        if not windows:
            return

        temporary = session.list_windows()[0]
        temporary.move_window(len(windows))

        for index, window in enumerate(windows):
            self.realize_window(session, window, index)

        temporary.kill_window()

    def realize_window(self, session, window_schema, index):
        window = session.new_window(
            window_name=window_schema.name,
            start_directory=window_schema.path,
            window_index=index,
            attach=index == 0)

        for option, value in window_schema.opts.items():
            window.set_window_option(option, value)

        pane = window.list_panes()[0]
        self.realize_items(pane, window_schema.items)

        return window

    def realize_items(self, pane, items):
        for item in items:
            if not item:
                continue
            if isinstance(item, Split):
                self.realize_split(pane, item)
            elif isinstance(item, six.string_types):
                pane.send_keys(item)

    def realize_split(self, previous, split_schema):
        pane = previous.split_window(
            start_directory=split_schema.path, vertical=split_schema.vertical)

        if split_schema.width is not None:
            pane.set_width(split_schema.width)
        if split_schema.height is not None:
            pane.set_height(split_schema.height)

        self.realize_items(pane, split_schema.items)


class TmuxSessionContext(context.Context):
    # https://libtmux.git-pull.com/en/latest/api.html#server-object
    tmux_server_kwargs = {}

    schema_realize_class = SchemaRealizer

    @property
    def session_name(self):
        return self.project_name

    @property
    def schema(self):
        return self.get_schema()

    def get_schema(self):
        return []

    @property
    def envs(self):
        return {}

    def get_commands(self):
        return super(TmuxSessionContext, self).get_commands() + [
            Status,
            Up,
            Kill,
            Attach,
            Inspect,
        ]


class _CollectByCall(object):
    def __init__(self, *args, **kwargs):
        self._items = []

    def __call__(self, *args):
        self._items += args
        return self

    @property
    def items(self):
        return self._items


class Window(_CollectByCall):
    def __init__(self, name=None, path=None, opts=None, **kwargs):
        super(Window, self).__init__()
        self.kwargs = kwargs
        self.name = name
        self.path = path
        self.opts = opts or {}


w = Window


class Split(_CollectByCall):
    def __init__(self,
                 path=None,
                 width=None,
                 height=None,
                 vertical=True,
                 **kwargs):
        super(Split, self).__init__()
        self.kwargs = kwargs
        self.path = path
        self.width = width
        self.height = height
        self.vertical = vertical


s = Split
v = Split
h = partial(Split, vertical=False)


class TmuxCommand(Command):
    @property
    def server(self):
        server = getattr(self.context, '_tmux_server', None)
        if server is None:
            server = libtmux.Server(**self.context.tmux_server_kwargs)
            setattr(self.context, '_tmux_server', server)
        return server

    def find_session_by_name(self, name):
        return self.server.find_where({
            'session_name': name,
        })

    @property
    def session(self):
        return self.find_session_by_name(self.context.session_name)

    def ensure_session_exists(self, expected=True):
        if bool(self.session) ^ expected:
            logger.error('Session %s', {
                False: 'exists',
                True: 'is not exists',
            }[expected])
            return False
        return True


class Up(TmuxCommand):
    short_help = 'Start session'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_session_exists(expected=False):
            return

        realizer = self.context.schema_realize_class()
        realizer.realize(self.server, self.context)


class Kill(TmuxCommand):
    short_help = 'Kill session'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_session_exists():
            return
        self.session.kill_session()


class Status(TmuxCommand):
    short_help = 'Session status'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.session is None:
            print('Session does not exist')
        else:
            print('Session exists')


class Attach(TmuxCommand):
    short_help = 'Attach session'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_session_exists():
            return
        self.session.attach_session()


class Inspect(TmuxCommand):
    usage = '<session>'
    short_help = 'Inspect session'

    weight = preset_base_command()

    template_window = 'w(name=\'{name}\', path=\'{path}\')(),'

    def invoke_with_args(self, args):
        def escape(value):
            return value.replace('\'', '\\\'')

        session_name = args['<session>']
        session = self.find_session_by_name(session_name)
        assert session, 'Session not found'

        print('session_name = \'{}\''.format(escape(session_name)))

        for window in session.list_windows():
            first_pane = window.list_panes()[0]

            name = window.get('window_name') or ''

            path = first_pane.get('pane_current_path')
            path = path.replace('\'', '\\\'')

            item = self.template_window.format(
                name=escape(name), path=escape(path))
            for line in item.splitlines():
                print(line)



from ds.presets.docker_base import DockerContext, PullContext, DefaultNaming
from ds.presets.docker_base.commands import Exec


class RedisContext(DockerContext):
    """
    https://hub.docker.com/_/redis/
    """

    def get_commands(self):
        return super(RedisContext, self).get_commands() + [
            Cli,
            Info,
        ]


class Context(DefaultNaming, RedisContext, PullContext):
    default_image = 'redis'


class Cli(Exec):
    def get_command(self):
        return 'redis-cli',


class Info(Cli):
    def get_command(self):
        return super(Info, self).get_command() + ('info', )



from ds.presets.tmux_template import TmuxSessionContext
from ds.presets.tmux_template import w
from ds.presets.tmux_template import v
from ds.presets.tmux_template import h


class TmuxTestContext(TmuxSessionContext):
    session_name = 'ds-tmux-test'

    def get_schema(self):
        return [
            w(name='test-a', path='/')(
                'date',
            ),
            w(name='test-b', path='/tmp/')(
                v(),
            ),
            w(name='test-c')(),
            w()(),
        ]


def test_context_without_class(capsys):
    context = TmuxTestContext()
    context.up()
    context.inspect(context.session_name)
    captured = capsys.readouterr()
    stdout = captured.out
    context.kill()

    assert stdout
    for window in context.get_schema():
        if not window.name:
            continue
        assert window.name in stdout



from pprint import pformat

from ds.utils import format_columns
from ds.utils import get_tty_width


class Summary(object):
    def __init__(self, title):
        self.title = title

    def render(self):
        pass


class TableSummary(Summary):
    def __init__(self, title, cells):
        self.cells = cells
        super(TableSummary, self).__init__(title)

    def render(self):
        return format_columns(*self.cells)


class FormatSummary(Summary):
    def __init__(self, title, data, **options):
        self.data = data
        self.options = options
        super(FormatSummary, self).__init__(title)

    def render(self):
        opts = self.options
        opts.setdefault('width', max([80, get_tty_width()]))
        return pformat(self.data, **opts)



from os.path import join

from ds.presets.docker_base import BaseDockerContext
from ds.presets.docker_base.commands import Exec, Shell


class PipMixin(BaseDockerContext):
    requirements_filename = 'requirements.txt'

    def get_pip_requirements_path(self):
        return join(
            self.project_root,
            self.requirements_filename,
        )

    def get_commands(self):
        return super(PipMixin, self).get_commands() + [
            Pip,
            PipFreeze,
        ]


class Pip(Exec):
    user = 0

    def get_command(self):
        return 'pip',


class PipFreeze(Shell):
    tty = False

    def get_command(self):
        return 'pip', 'freeze',

    def invoke_with_args(self, args):
        super(PipFreeze, self).invoke_with_args(args)
        result = self.context.executor.commit()
        path = self.context.get_pip_requirements_path()
        with open(path, 'w') as output:
            output.write(result.stdout)
        self.context.executor.append(('git', 'diff', path))



from ds.presets.docker_base import mixins
from ds.presets.docker_base import naming
from ds.presets.docker_base import DockerContext
from ds.presets.docker_base import PullContext
from ds.presets.docker_base.commands import Exec
from ds.command import preset_command


class ClojureContext(DockerContext):
    def get_commands(self):
        return super(ClojureContext, self).get_commands() + [
            NewProject,
            Lein,
            Repl,
        ]


class Context(ClojureContext, naming.DefaultNaming, mixins.UserMixin,
              mixins.HomeMountsMixin, mixins.ProjectMountMixin,
              PullContext):
    default_image = 'clojure'
    default_tag = 'lein'
    container_home = '/tmp',

    def get_environment(self):
        environment = super(Context, self).get_environment()
        environment.setdefault('HOME', self.container_home[0])
        return environment

    def get_container_default(self):
        return 'lein', 'repl'


class Lein(Exec):
    weight = preset_command()

    def get_command(self):
        return 'lein',


class Repl(Exec):
    weight = preset_command()

    def get_command(self):
        return 'lein', 'repl',


class NewProject(Exec):
    usage = '[<template>] [<name>]'
    short_help = 'Generate new project'
    consume_all_args = False

    weight = preset_command()

    def format_args(self, args):
        to_dir = ()
        if self.context.working_dir:
            to_dir = '--to-dir', self.context.working_dir
        return 'lein', 'new', \
                args.get('<template>', None) or 'app', \
                args.get('<name>', None) or 'app', \
                to_dir, '--force',



from __future__ import print_function
from __future__ import unicode_literals
from logging import getLogger
from weakref import ref

from docopt import docopt
from six import with_metaclass

from ds import path
from ds.environment import get_environment
from ds.utils import pretty_print_object
from ds.utils import numerator
from ds.utils import is_interactive
from ds.discover import find_contexts
from ds.utils.kebab_to_snake import kebab_to_snake


logger = getLogger(__name__)


generic_command = numerator(1000)
preset_base_command = numerator(2000)
preset_command = numerator(3000)


class CommandMeta(type):
    pass


class BaseCommand(with_metaclass(CommandMeta)):
    name = None

    hidden = False

    usage = ''
    short_help = ''

    weight = 0

    def __init__(self, context):
        self._context = ref(context)

    @property
    def context(self):
        return self._context()


class Command(BaseCommand):
    options_first = True
    consume_all_args = False

    def get_name(self):
        return kebab_to_snake(self.__class__.__name__)

    def parse_command_line(self, command_line):
        command_line = command_line or ()
        if self.consume_all_args:
            return command_line
        usage = 'usage: {name} {rest}'.format(rest=self.usage, name=self.get_name())
        return docopt(usage, argv=command_line, options_first=self.options_first)

    def invoke_with_args(self, args):
        raise NotImplementedError

    def invoke(self, command_line=None, args=None):
        assert (bool(command_line) != bool(args)) or not command_line
        if args:
            return self.invoke_with_args(args)
        return self.invoke_with_args(self.parse_command_line(command_line))

    __call__ = invoke


class SwitchContext(Command):
    options_first = True
    usage = '[<context>]'
    short_help = 'Switch context'

    hidden = True

    weight = generic_command()

    def invoke_with_args(self, args):
        name = self.get_context_name(args)
        if name:
            get_environment().set('context', name)

    def get_context_name(self, args):
        src = args.get('<context>', None)
        if src:
            return src

        if not is_interactive():
            logger.error('Context name is required for non-interactive shell')
            return

        return self.ask_for_context()

    def ask_for_context(self):
        variants = [' '.join(item) for item in find_contexts()]
        preset = self.context.executor. \
            fzf(variants, prompt='Context')
        if not preset:
            return
        return preset.split(' ', 1)[0]


class ListCommands(Command):
    short_help = 'List all commands in context (autocomplete helper)'
    hidden = True

    weight = generic_command()

    def invoke_with_args(self, args):
        print(' '.join(self.context.commands.keys()))


class ShowContext(Command):
    short_help = 'Show a context info (autocomplete helper)'
    hidden = True

    weight = generic_command()

    def invoke_with_args(self, args):
        context_class = self.context.__class__
        print('Context:', context_class, context_class.__module__)
        print('Additional imports:', ';'.join(path.get_additional_import()))
        pretty_print_object(self.context)


class EditContext(Command):
    short_help = 'Edit a context with $EDITOR'
    hidden = True

    weight = generic_command()

    def invoke_with_args(self, args):
        if not is_interactive():
            return
        self.context.executor.edit_file(self.context.source_file)


class DsRepl(Command):
    short_help = 'Read-eval-print-loop'
    hidden = True

    weight = generic_command()

    def invoke_with_args(self, args):
        self.context.repl_class(self.context)(args)



import pytest

from ds.environment import get_environment


context = 'plain-context'


def test_switch_context(shell):
    shell.call_ds('switch-context', context)
    assert context == get_environment().get('context')


def test_additional_summary(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('-h')
    assert result.code == 0
    assert 'Test:\n a  1\n b  2' in result.stdout


def test_commands_list(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('-h')
    assert result.code == 0


@pytest.mark.parametrize('what', [
    None,
    '',
    'test',
    'with space',
])
def test_invoke_command(shell, what):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('echo-test', what)
    assert result.code == 0
    assert result.stdout.strip() == (what or '')



from os import makedirs
from os.path import exists
from os.path import dirname
from os.path import join
import json
from logging import getLogger

from cachetools import cached

from ds.path import pwd


logger = getLogger(__name__)


NULL = object()


class BaseEnvironment(object):
    def __init__(self):
        self._data = None

    @property
    @cached({})
    def environment_filename(self):
        result = self.get_environment_filename()
        return result

    def get_environment_filename(self):
        raise NotImplementedError

    def load(self):
        filename = self.environment_filename
        if not exists(filename):
            self._data = {}
            return
        with open(filename, 'r') as file:
            try:
                self._data = json.load(file)
            except ValueError:
                self._data = {}

    def save(self):
        filename = self.environment_filename
        logger.debug('Save data to %s', filename)
        path = dirname(filename)
        if not exists(path):
            makedirs(path)
        with open(filename, 'w') as file:
            json.dump(self._data, file)

    @property
    def data(self):
        if self._data is None:
            self.load()
        return self._data

    def get(self, key, default=None):
        value = self.data.get(key, NULL)
        if value is not NULL:
            return value
        if callable(default):
            value = default()
        else:
            value = default
        self.set(key, value)
        return value

    def set(self, key, value):
        self.data[key] = value
        self.save()

    def invalidate(self):
        self._data = None


class PwdEnvironment(BaseEnvironment):
    env_filename = '.ds-env'

    def get_environment_filename(self):
        return join(pwd(), self.env_filename)


@cached({})
def get_environment():
    return PwdEnvironment()



from ds.environment import get_environment


context = 'dsjk'


def test_switch_context(shell):
    shell.call_ds('switch-context', context)
    assert context == get_environment().get('context')


def test_show_context(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('show-context')
    assert result.code == 0


def test_edit_context(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('edit-context')
    assert result.code == 0


def test_list_context(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('list-contexts')
    assert result.code == 0


def test_install_autocomplete(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('install-autocomplete')
    assert result.code == 0



from ds import errors


context = 'empty-context'


def test_context_without_class(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('-h')
    assert result.code == errors.NO_CONTEXT_CLASS


def test_fallback(shell):
    shell.call_ds('switch-context', context)
    result = shell.call_ds('-h')
    assert result.code == errors.NO_CONTEXT_CLASS
    result = shell.call_ds('-h')
    assert result.code == 0



def test_docker_show_run_options():
    pass


def test_docker_create():
    pass


def test_docker_start():
    pass


def test_docker_stop():
    pass


def test_docker_restart():
    pass


def test_docker_recreate():
    pass


def test_docker_kill():
    pass


def test_docker_rm():
    pass


def test_docker_logs():
    pass


def test_docker_inspect():
    pass


def test_docker_attach():
    pass


def test_docker_exec():
    pass


def test_docker_shell():
    pass


def test_docker_root_shell():
    pass


def test_docker_pull():
    pass


def test_docker_build():
    pass



from os import chdir
from os import getcwd
from contextlib import contextmanager


@contextmanager
def cd(path, **opts):
    previous = getcwd()
    try:
        chdir(path)
        yield
    finally:
        chdir(previous)
        finalizers = opts.get('finalizers', None) or []
        for finalizer in finalizers:
            finalizer()



from __future__ import unicode_literals
from logging import getLogger
from pprint import pprint
import sys

from ds.command import Command
from ds.command import preset_base_command
from ds.utils.term import get_tty_width


logger = getLogger(__name__)


class DockerCommand(Command):
    container_name_required = False
    image_name_required = False

    weight = preset_base_command()

    @property
    def is_exists(self):
        return self.context.container is not None

    @property
    def is_running(self):
        return self.is_exists and \
               self.context.container.status == 'running'

    def ensure_running(self):
        if not self.is_running:
            logger.error('Container is not running')
            return
        return True


class Pull(DockerCommand):
    image_name_required = True

    short_help = 'Pull an image from a registry'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.executor.append([
            ('docker', 'pull'),
            self.context.image_name,
        ])


class Build(DockerCommand):
    image_name_required = True

    short_help = 'Build an image'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.executor.append([
            ('docker', 'build'),
            ('-t', self.context.image_name),
            ('-f', self.context.docker_file) if self.context.docker_file else (),
            self.context.build_path if self.context.build_path else (),
        ])


class ShowRunOptions(DockerCommand):
    container_name_required = True
    image_name_required = True

    usage = '[<args>...]'
    short_help = ''
    consume_all_args = True

    hidden = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        options = self.context. \
            get_run_options(image=self.context.image_name,
                            name=self.context.container_name,
                            command=args if args else None)
        pprint(options, width=get_tty_width())


class Create(DockerCommand):
    container_name_required = True
    image_name_required = True

    usage = '[<args>...]'
    short_help = 'Create a container'
    consume_all_args = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.is_exists:
            logger.error('Container exists already')
            sys.exit(1)

        command = list(self.context.container_entry) + \
                  list(args or self.context.container_cmd)

        options = self.context. \
            get_run_options(image=self.context.image_name,
                            name=self.context.container_name,
                            command=command if command else None)

        if self.is_exists:
            if self.context.remove_before_start:
                logger.debug('Remove a container')
                self.context.rm()
            else:
                logger.error('Container exists')
                return options, self.context.container

        logger.debug('Create a container with %s', options)

        container = self.context.client.containers.create(**options)
        logger.debug('%s created', container.id)

        return options, container


class Start(DockerCommand):
    container_name_required = True
    image_name_required = True

    usage = '[<args>...]'
    short_help = 'Start a container'
    consume_all_args = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.is_running:
            if self.context.stop_before_start:
                logger.debug('Store a container')
                self.context.stop()
            else:
                logger.error('Container is running already')
                sys.exit(1)

        options, container = self.context.create(args)

        if options.get('detach', False):
            container.start()
            self.context.logs()
        else:
            self.context.attach()


class Stop(DockerCommand):
    container_name_required = True

    short_help = 'Stop a container'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.is_running:
            logger.error('Container is not working')
            return
        logger.error('Stop a container')
        self.context.container.stop()


class Restart(DockerCommand):
    container_name_required = True
    image_name_required = True

    short_help = 'Restart a container'
    usage = '[<args>...]'
    consume_all_args = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.is_running:
            self.context.stop()
        self.context.start(args)


class Recreate(DockerCommand):
    container_name_required = True
    image_name_required = True

    short_help = 'Recreate a container'
    usage = '[<args>...]'
    consume_all_args = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.is_running:
            self.context.stop()
        if self.is_exists:
            self.context.rm()
        self.context.start(args)


class Kill(DockerCommand):
    container_name_required = True

    short_help = 'Kill a container'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.context.container:
            self.context.container.kill()


class Rm(DockerCommand):
    container_name_required = True

    short_help = 'Remove a container'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if self.is_running:
            self.context.stop()
        if self.context.container:
            self.context.container.remove()


class Logs(DockerCommand):
    container_name_required = True

    short_help = 'Fetch the logs of a container'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_running():
            return
        self.context.executor.append([
            ('docker', 'logs'),
            '--follow' if self.context.logs_follow else (),
            ('--tail', str(self.context.logs_tail)),
            self.context.container_name,
        ])


class Inspect(DockerCommand):
    container_name_required = True

    short_help = 'Return low-level information about a container'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_running():
            return
        self.context.executor.append([
            ('docker', 'inspect'),
            self.context.container_name,
        ])


class Attach(DockerCommand):
    container_name_required = True

    short_help = 'Attach a local stdin/stdout to a container'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.is_exists:
            logger.error('Container doesn\'t exist')
            return

        print('Note: Press {} to detach'.format(self.context.detach_keys))

        if self.is_running:
            command = 'attach'
        else:
            command = ('start', '--attach', '--interactive')

        self.context.executor.append([
            'docker',
            command,
            ('--detach-keys', self.context.detach_keys),
            self.context.container_name,
        ])


class Exec(DockerCommand):
    container_name_required = True

    usage = 'usage: {name} <args>...'
    consume_all_args = True

    interactive = True
    tty = True

    weight = preset_base_command()

    @property
    def user(self):
        from .mixins import UserMixin
        user = None
        if isinstance(self.context, UserMixin):
            user = self.context.container_user
        return user

    @property
    def short_help(self):
        args = self.get_command()
        if args:
            return '`{}`'.format(' '.join(args))
        return 'Run a command in a container'

    def invoke_with_args(self, invoke_args):
        if not self.is_running:
            logger.error('Container isn\'t running')
            return

        args = self.format_args(invoke_args)

        self.context.executor.append([
            ('docker', 'exec'),
            '-i' if self.interactive else (),
            '-t' if self.tty else (),
            ('-u', str(self.user)) if self.user is not None else (),
            self.context.container_name,
            args,
        ])

    def format_args(self, invoke_args):
        result = list(self.get_command()) + list(invoke_args)
        assert result, 'No arguments for exec'
        return result

    def get_command(self):
        return ()


class Shell(Exec):
    container_name_required = True

    weight = preset_base_command()

    user = None

    @property
    def short_help(self):
        shell = self.context.shell_entry
        user = self.user
        if user is None:
            user = self.context.container_user
        return 'Run {} in a container with uid={}'.\
            format(shell, user if user is not None else '*unfilled*')

    def get_command(self):
        return self.context.shell_entry,


class RootShell(Shell):
    container_name_required = True

    weight = preset_base_command()

    user = 0



# encoding: utf-8

NO_CONTEXT_MODULE = 1
NO_CONTEXT_CLASS = 2
NO_CONTEXT_CLASS_MESSAGE = 'Context module should contain a `Context` class'
COMMAND_NOT_FOUND = 3
COMMAND_NOT_FOUND_MESSAGE = 'Command not found'



from __future__ import unicode_literals
from collections import OrderedDict
from logging import getLogger

from ds import executor
from ds import command
from ds import path
from ds.summary import TableSummary as _TableSummary
from ds.utils import cd


logger = getLogger(__name__)


class BaseContext(object):
    def __init__(self, **options):
        self._commands = OrderedDict()

        for command_class in self.get_commands():
            if not command_class:
                continue
            name = None
            if isinstance(command_class, (tuple, list)):
                name, command_class = command_class
            command = command_class(self)
            if name is None:
                name = command.get_name()
            self._commands[name] = command

        # self._commands = OrderedDict([])
        # self._sort_commands(self._filter_commands(self._commands.items()))
        # commands = self._filter_commands(commands)
        # commands = self._sort_commands(commands)

    @property
    def commands(self):
        return self._commands

    def _filter_commands(self, commands):
        return commands

    def _sort_commands(self, commands):
        key = lambda command: (command.weight, command.__name__)
        return sorted(commands, key=key)

    def get_commands(self):
        return []

    def get_command(self, name):
        if self.commands is None:
            return
        for candidate in (name, name.replace('_', '-')):
            if candidate in self.commands:
                return self.commands[candidate]

    def __getitem__(self, key):
        command = self.get_command(key)
        if command:
            return command
        raise KeyError

    def __getattribute__(self, name):
        try:
            return super(BaseContext, self).__getattribute__(name)
        except AttributeError:
            command = self.get_command(name)
            if command:
                return command
            raise

    def check(self):
        pass


class IntrospectionMixin(BaseContext):
    @property
    def source_file(self):
        from inspect import getmodule
        from inspect import getsourcefile
        return getsourcefile(getmodule(self))

    def get_commands(self):
        return super(IntrospectionMixin, self).get_commands() + [
            command.ShowContext,
            command.EditContext,
        ]


class ChangeDirMixin(BaseContext):
    def cd(self, path):
        return cd(path, finalizers=(self.executor.commit, ))


class ExecutorMixin(BaseContext):
    executor_class = executor.Executor

    def __init__(self, **options):
        self._executor = self.executor_class()
        super(ExecutorMixin, self).__init__(**options)

    @property
    def executor(self):
        return self._executor


class TestExecutorMixin(ExecutorMixin):
    executor_class = executor.TestExecutor


class ReplMixin(BaseContext):
    @property
    def repl_class(self):
        from ds.repl import Repl
        return Repl

    def get_commands(self):
        return super(ReplMixin, self).get_commands() + [
            command.DsRepl,
        ]


class ProjectMixin(BaseContext):
    def get_project_root(self):
        return path.get_project_root()

    @property
    def project_root(self):
        return self.get_project_root()

    def get_project_name(self):
        return path.get_project_name()

    @property
    def project_name(self):
        return self.get_project_name()


class Context(ProjectMixin, IntrospectionMixin, ReplMixin, ExecutorMixin,
              ChangeDirMixin, BaseContext):
    def get_commands(self):
        return super(Context, self).get_commands() + [
            command.ListCommands,
            command.SwitchContext,
        ]

    def get_additional_summary(self):
        name = '{} ({})'.format(self.__class__.__module__, self.source_file)
        project = '{} ({})'.format(self.project_name, self.project_root)
        return [
            _TableSummary('Context', [['Name', name], ['Project', project]]),
        ]



# pylint: disable=unused-import
from tmux_base import TmuxSessionContext
from tmux_base import v
from tmux_base import h
from tmux_base import w


class Context(TmuxSessionContext):
    def get_schema(self):
        return [
            w(name='first')(
                'uname -a',
            ),
            w(name='second', path='/tmp/')(
                'date',
                'htop',
            ),
            w()(
                'echo "hi"',
                v(path='/var/tmp')(
                    'l',
                    'date',
                    h(
                        'echo "horizontal split 1"',
                    ),
                    h(
                        'echo "horizontal split 2"',
                    ),
                ),
            ),
        ]



import subprocess


def get_tty_size(fallback=None):
    try:
        return map(int, subprocess.check_output(['stty', 'size']).split())
    except:
        pass
    return fallback or (None, None)


def get_tty_width():
    _, width = get_tty_size()
    return width



from __future__ import absolute_import
from __future__ import print_function
from __future__ import unicode_literals

from logging import getLogger

from ds.command import Command
from ds.command import preset_base_command
from ds.utils.flatten import flatten


logger = getLogger()


class CreateDisk(Command):
    usage = '[--format=<format>] [--size=<size>] [<name>]'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        size = args.get('--size', None) or '1G'
        format_ = args.get('--format', None) or 'qcow2'
        name = args.get('<name>', None)
        if not name:
            name = 'disk-{}-{}.{}'.format(self.context.project_name or name,
                                          size, format_)
        self.context.executor.append(flatten((
            self.context.qemu_img,
            'create',
            ('-f', format_),
            name,
            size,
        )))


class RunIso(Command):
    usage = '<iso> [<args>...]'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.start(('-cdrom', args.get('<iso>'), args.get('<args>')))


class RunDisk(Command):
    usage = '<disk> [<args>...]'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.start(('-hda', args.get('<disk>'), args.get('<args>')))


class RunIsoAndDisk(Command):
    usage = '--iso=<iso> --disk=<disk> [<args>...]'

    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.start(('-hda', args.get('--disk'),
                            '-cdrom', args.get('--iso'),
                            args.get('<args>')))


class Start(Command):
    consume_all_args = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.executor.append(flatten((
            self.context.qemu_bin,
            self.context.get_run_options(),
            args,
        )))
        if self.context.vnc:
            self.context.viewer()


class Viewer(Command):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        display = self.context.qemu_environment.get('display')
        assert display

        address = '{host}:{display}'.format(host=self.context.vnc_host,
                                            display=5900 + display)

        self.context.executor.append(flatten((
            self.context.viewer_bin,
            address,
        )))


class AttachTelnet(Command):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        telnet = self.context.telnet
        if not telnet:
            logger.error('Context option `telnet` is not defined')
            return
        self.context.connect_to(telnet)


class AttachShell(Command):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        shell = self.context.shell
        if not shell:
            logger.error('Context option `shell` is not defined')
            return
        self.context.connect_to(shell)


class _MonitorCommand(Command):
    def ensure_monitor(self):
        if not self.context.monitor:
            logger.error('Context option `monitor` is not defined')
            return
        return True


class AttachMonitor(_MonitorCommand):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_monitor():
            return
        self.context.connect_to(self.context.monitor)


class Stop(_MonitorCommand):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_monitor():
            return
        self.context.send_to(self.context.monitor, 'quit')


class Reset(_MonitorCommand):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.send_to(self.context.monitor, 'system_reset')


class Info(_MonitorCommand):
    consume_all_args = True

    weight = preset_base_command()

    def invoke_with_args(self, args):
        if not self.ensure_monitor():
            return
        self.context.send_to(self.context.monitor, ' '.join([
            'info',
            ' '.join(args),
        ]) + '\n')


class Kill(Command):
    weight = preset_base_command()

    def invoke_with_args(self, args):
        self.context.executor.append(('ps', 'ax'))
        result = self.context.executor.commit()
        if result.code:
            logger.error('Can\'t get a process list')
            return

        stdout = result.stdout.decode('utf-8').split('\n')
        for item in stdout:
            is_qemu = self.context.qemu_bin in item and \
                      '-name {}'.format(self.context.name) in item
            if not is_qemu:
                continue

            pid = item.split(' ', 1)[0]
            self.context.executor.append(('kill', pid))
            return

        logger.error('Process is not found')



def flatten(args):
    result = []
    for item in args:
        if isinstance(item, (set, tuple, list)):
            result.extend(flatten(item))
        else:
            result.append(item)
    return result



from __future__ import unicode_literals
from __future__ import print_function
import sys
from logging import getLogger

from prompt_toolkit import PromptSession
from prompt_toolkit.styles import Style
from prompt_toolkit.completion import WordCompleter

from ds.utils import flatten
from ds.utils import drop_empty


logger = getLogger(__name__)


class Repl(object):
    RELOAD, QUIT = range(1, 3)
    repl_actions = {
        'r': RELOAD,
        'reload': RELOAD,
        'q': QUIT,
        'quit': QUIT,
    }

    def __init__(self, context):
        self.context = context

    def __call__(self, args):
        completer = self.completer()
        style = self.style()

        session = PromptSession(
            message='> ',
            bottom_toolbar=self.bottom_toolbar,
            vi_mode=True,
            complete_while_typing=True,
            completer=completer,
            style=style,
        )

        while True:
            try:
                input_ = session.prompt()
            except KeyboardInterrupt:
                continue
            except EOFError:
                break

            try:
                self.process_input(input_)
            except KeyboardInterrupt:
                pass

        logger.info('Done')

    def call_repl(self, input_):
        repl_action = self.repl_actions.get(input_, None)

        if repl_action == self.RELOAD:
            self.context.executor.append(sys.argv)
            self.context.executor.commit(replace=True)

        elif repl_action == self.QUIT:
            sys.exit()

        else:
            logger.error('Unknown repl command')

    def call_shell(self, input_):
        args = flatten([
            '/bin/bash',
            '-c',
            input_,
        ])
        self.context.executor.\
            append(args, skip_stdin=True, skip_stdout=True, skip_stderr=True)
        self.context.executor.commit()

    def call_context(self, input_):
        args = flatten([sys.argv[:-1], input_])
        self.context.executor.\
            append(args, skip_stdin=True, skip_stdout=True, skip_stderr=True)
        self.context.executor.commit()

    def process_input(self, input_):
        if not input_:
            return

        if input_.startswith(':'):
            return self.call_repl(input_[1:])

        elif input_.startswith('!'):
            return self.call_shell(input_[1:])

        return self.call_context(input_)

    def bottom_toolbar(self):
        return ' '.join(drop_empty(
            self.context.project_name,
            self.context.project_root,
            self.context.source_file,
        ))

    def completer(self):
        variants = []
        # variants += list(map(six.u, self.context.commands.keys()))
        variants += list(self.context.commands.keys())
        variants += list(self.repl_actions.keys())
        variants += ['-h', ]
        return WordCompleter(variants)

    def style(self):
        return Style.from_dict({
            'completion-menu.completion': 'bg:#008888 #ffffff',
            'completion-menu.completion.current': 'bg:#00aaaa #000000',
            'scrollbar.background': 'bg:#88aaaa',
            'scrollbar.button': 'bg:#222222',
        })



import os
from os.path import exists
from os.path import expanduser
from os.path import join

from docker.types import Mount

from .base import BaseDockerContext
from . import commands


class AttachMixin(BaseDockerContext):
    detach_keys = 'ctrl-c'

    def get_commands(self):
        return super(AttachMixin, self).get_commands() + [
            commands.Attach,
        ]


class ManageContainerMixin(BaseDockerContext):
    stop_before_start = True
    remove_before_start = True

    def get_commands(self):
        return super(ManageContainerMixin, self).get_commands() + [
            commands.Start,
            commands.Stop,
            commands.Restart,
            commands.Kill,
            commands.Rm,
            commands.Inspect,
            commands.Exec,
        ]


class CreateContainerMixin(ManageContainerMixin):
    def get_commands(self):
        return super(CreateContainerMixin, self).get_commands() + [
            commands.ShowRunOptions,
            commands.Create,
            commands.Recreate,
        ]

    @property
    def container_entry(self):
        return self.get_container_entry()

    @property
    def container_cmd(self):
        return self.get_container_cmd()

    def get_container_entry(self):
        return []

    def get_container_cmd(self):
        return []


class LogsMixin(BaseDockerContext):
    logs_tail = 100
    logs_follow = True

    def get_commands(self):
        return super(LogsMixin, self).get_commands() + [
            commands.Logs,
        ]


class ContainerUserMixin(BaseDockerContext):
    container_user = None

    def get_run_options(self, **options):
        options.setdefault('user', self.container_user)
        return super(ContainerUserMixin, self).get_run_options(**options)


class NetworkMixin(ContainerUserMixin):
    network = None

    def get_run_options(self, **options):
        options.setdefault('network', self.network)
        return super(NetworkMixin, self).get_run_options(**options)


class UserMixin(ContainerUserMixin):
    @property
    def container_user(self):
        return os.getuid()


class MountsMixin(BaseDockerContext):
    def get_run_options(self, **options):
        options.setdefault('mounts', self.get_mounts())
        return super(MountsMixin, self).get_run_options(**options)

    def get_mounts(self):
        return []


class EnvironmentMixin(BaseDockerContext):
    container_environment = {}

    def get_run_options(self, **options):
        options.setdefault('environment', self.get_environment())
        return super(EnvironmentMixin, self).get_run_options(**options)

    def get_environment(self):
        return self.container_environment


class HomeMountsMixin(MountsMixin):
    container_home = '/',

    home_mounts = [
        '.bashrc',
        '.inputrc',
        '.config/bash',
        '.psqlrc',
        '.liquidpromptrc',
    ]

    def get_mounts(self):
        additional = []
        for src in self.home_mounts:
            full_src = expanduser(join('~', src))
            if not exists(full_src):
                continue
            for container_home in self.container_home:
                full_dest = join(container_home, src)
                mount = Mount(target=full_dest,
                              source=full_src,
                              type='bind',
                              read_only=True)
                additional.append(mount)
        return super(HomeMountsMixin, self).get_mounts() + additional


class WorkingDirMixin(MountsMixin):
    working_dir = '/app/'

    def get_run_options(self, **options):
        options.setdefault('working_dir', self.working_dir)
        return super(WorkingDirMixin, self).get_run_options(**options)


class ProjectMountMixin(WorkingDirMixin):
    @property
    def project_mount_path(self):
        return self.project_root

    @property
    def project_mount_readonly(self):
        return False

    def get_mounts(self):
        mount = Mount(target=self.working_dir,
                      source=self.project_mount_path,
                      type='bind',
                      read_only=self.project_mount_readonly)
        return super(ProjectMountMixin, self).get_mounts() + [mount]


class ShellMixin(ContainerUserMixin):
    shell_entry = '/bin/bash'

    def get_commands(self):
        return super(ShellMixin, self).get_commands() + [
            commands.Shell,
            commands.RootShell,
        ]



from setuptools import find_packages
from setuptools import setup

import ds

setup(
    name='dsjk',
    version=ds.__version__,
    author='',
    author_email='',
    url='',
    description='',
    long_description=open('README.md').read().strip(),
    packages=find_packages(),
    install_requires=[
        'six',
        'docopt',
        'cachetools',
    ],
    extras_require={
        'repl': ['prompt_toolkit>=2.0.0'],
        'tmux': ['libtmux'],
        'docker': ['docker>=3.1.3'],
    },
    test_suite='tests',
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'ds=ds.__main__:main',
        ],
    },
)



from collections import OrderedDict
from pprint import pformat
from logging import getLogger


logger = getLogger(__name__)


def pretty_print_object(instance):
    for key in dir(instance):
        if key.startswith('_'):
            continue
        try:
            value = getattr(instance, key)
        except:
            logger.error('Error while getting value of "%s"', key)
            continue
        if callable(value):
            continue
        if isinstance(value, OrderedDict):
            value = dict(value)
        formatted_value = pformat(value, indent=1)
        sep = {
            False: ' ',
            True: '\n',
        }['\n' in formatted_value]
        print((':' + sep).join([key, formatted_value]))



from __future__ import unicode_literals
import os

from ds.presets.docker_base import PullContext
from ds.presets.docker_base import DefaultNaming
from ds.presets.docker_base import UserMixin
from ds.presets.docker_base import HomeMountsMixin
from ds.presets.docker_base import ProjectMountMixin


class Context(ProjectMountMixin, UserMixin, HomeMountsMixin,
              DefaultNaming, PullContext):
    @property
    def default_image(self):
        return os.environ.get('IMAGE', 'debian')



from __future__ import unicode_literals
import sys

try:
    import docker
    import docker.errors
    from docker.types import Mount
except ImportError:
    print('Install docker-py with `pip install docker`')
    sys.exit(1)

from ds.summary import TableSummary
from .base import BaseDockerContext
from . import commands
from . import naming
from . import mixins


UNDEFINED = '(undefined)'


class DockerContext(mixins.MountsMixin, mixins.EnvironmentMixin,
                    mixins.NetworkMixin, mixins.ShellMixin,
                    mixins.LogsMixin, mixins.AttachMixin,
                    mixins.ManageContainerMixin,
                    BaseDockerContext):
    def get_additional_summary(self):
        container = self.container

        container_name = '-'
        if self.has_container_name:
            container_name = self.container_name
        image_name = '-'
        if self.has_image_name:
            image_name = self.image_name

        cells = [
            ['Name', container_name or UNDEFINED],
            ['Image', image_name or UNDEFINED],
            ['Status', container.status if container else '-'],
            ['ID', container.short_id if container else '-'],
        ]

        return super(DockerContext, self).get_additional_summary() + [
            TableSummary('Container', cells),
        ]


class ExternalContext(DockerContext):
    pass


class BuildContext(naming.BuildNaming, mixins.CreateContainerMixin,
                   DockerContext):
    def get_commands(self):
        return super(BuildContext, self).get_commands() + [
            commands.Build,
        ]

    def get_build_path(self):
        return self.project_root

    @property
    def build_path(self):
        return self.get_build_path()

    def get_docker_file(self):
        return 'Dockerfile'

    @property
    def docker_file(self):
        return self.get_docker_file()


class PullContext(naming.ImageNaming, mixins.CreateContainerMixin,
                  DockerContext):
    default_image = None
    default_tag = 'latest'

    def check(self):
        assert self.default_image, 'Default image is not set'
        super(PullContext, self).check()

    @property
    def image_name(self):
        if not self.default_image:
            return
        return ':'.join(filter(lambda value: value,
                               [self.default_image, self.default_tag]))

    def get_commands(self):
        return super(PullContext, self).get_commands() + [
            commands.Pull,
        ]
